@@ -943,3 +943,14 @@ var wakaEnv;
  * @suppress {undefinedVars}
  */
 var wakaBuffer;
+
+
+// Browser externs on global window object.
+var pageXOffset;
+var pageYOffset;
+var innerWidth;
+var innerHeight;
+var outerWidth;
+var outerHeight;
+var event;
+var devicePixelRatio;
