@@ -116,6 +116,9 @@ var SyscallsLibrary = {
       var node;
       var lookup = FS.lookupPath(path, { follow: true });
       node = lookup.node;
+      if (!node) {
+        return -{{{ cDefine('ENOENT') }}};
+      }
       var perms = '';
       if (amode & {{{ cDefine('R_OK') }}}) perms += 'r';
       if (amode & {{{ cDefine('W_OK') }}}) perms += 'w';
@@ -259,7 +262,7 @@ var SyscallsLibrary = {
 #endif
   ],
   _emscripten_syscall_munmap: function(addr, len) {
-    if (addr == {{{ cDefine('MAP_FAILED') }}} || len == 0) {
+    if (addr === {{{ cDefine('MAP_FAILED') }}} || len === 0) {
       return -{{{ cDefine('EINVAL') }}};
     }
     // TODO: support unmmap'ing parts of allocations
@@ -267,7 +270,7 @@ var SyscallsLibrary = {
     if (!info) return 0;
     if (len === info.len) {
       var stream = FS.getStream(info.fd);
-      SYSCALLS.doMsync(addr, stream, len, info.flags)
+      SYSCALLS.doMsync(addr, stream, len, info.flags);
       FS.munmap(stream);
       SYSCALLS.mappings[addr] = null;
       if (info.allocated) {
@@ -300,7 +303,7 @@ var SyscallsLibrary = {
 #endif // SYSCALLS_REQUIRE_FILESYSTEM
   },
   __syscall5: function(which, varargs) { // open
-    var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get() // optional TODO
+    var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get(); // optional TODO
     var stream = FS.open(pathname, flags, mode);
     return stream.fd;
   },
@@ -757,8 +760,8 @@ var SyscallsLibrary = {
       });
     });
 #else
-#if BYSYNCIFY
-    return Bysyncify.handleSleep(function(wakeUp) {
+#if WASM_BACKEND && ASYNCIFY
+    return Asyncify.handleSleep(function(wakeUp) {
       var mount = stream.node.mount;
       if (!mount.type.syncfs) {
         // We write directly to the file system, so there's nothing to do here.
@@ -778,7 +781,7 @@ var SyscallsLibrary = {
       return -stream.stream_ops.fsync(stream);
     }
     return 0; // we can't do anything synchronously; the in-memory FS is already synced to
-#endif // BYSYNCIFY
+#endif // WASM_BACKEND && ASYNCIFY
 #endif // EMTERPRETIFY_ASYNC
   },
   __syscall121: function(which, varargs) { // setdomainname
@@ -930,39 +933,6 @@ var SyscallsLibrary = {
     var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
     return SYSCALLS.doReadv(stream, iov, iovcnt);
   },
-#if SYSCALLS_REQUIRE_FILESYSTEM == 0
-  $flush_NO_FILESYSTEM: function() {
-    // flush anything remaining in the buffers during shutdown
-    var fflush = Module["_fflush"];
-    if (fflush) fflush(0);
-    var buffers = SYSCALLS.buffers;
-    if (buffers[1].length) SYSCALLS.printChar(1, {{{ charCode("\n") }}});
-    if (buffers[2].length) SYSCALLS.printChar(2, {{{ charCode("\n") }}});
-  },
-  __syscall146__deps: ['$flush_NO_FILESYSTEM'],
-#if EXIT_RUNTIME == 1 && !MINIMAL_RUNTIME // MINIMAL_RUNTIME does not have __ATEXIT__ (so it does not get flushed stdout at program exit - programs in MINIMAL_RUNTIME do not have a concept of exiting)
-  __syscall146__postset: '__ATEXIT__.push(flush_NO_FILESYSTEM);',
-#endif
-#endif
-  __syscall146: function(which, varargs) { // writev
-#if SYSCALLS_REQUIRE_FILESYSTEM
-    var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
-    return SYSCALLS.doWritev(stream, iov, iovcnt);
-#else
-    // hack to support printf in SYSCALLS_REQUIRE_FILESYSTEM=0
-    var stream = SYSCALLS.get(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
-    var ret = 0;
-    for (var i = 0; i < iovcnt; i++) {
-      var ptr = {{{ makeGetValue('iov', 'i*8', 'i32') }}};
-      var len = {{{ makeGetValue('iov', 'i*8 + 4', 'i32') }}};
-      for (var j = 0; j < len; j++) {
-        SYSCALLS.printChar(stream, HEAPU8[ptr+j]);
-      }
-      ret += len;
-    }
-    return ret;
-#endif // SYSCALLS_REQUIRE_FILESYSTEM
-  },
   __syscall147__deps: ['$PROCINFO'],
   __syscall147: function(which, varargs) { // getsid
     var pid = SYSCALLS.get();
@@ -1457,6 +1427,47 @@ var SyscallsLibrary = {
 #endif
     return 0;
   },
+
+  // WASI (WebAssembly System Interface) call support
+#if SYSCALLS_REQUIRE_FILESYSTEM == 0
+  $flush_NO_FILESYSTEM: function() {
+    // flush anything remaining in the buffers during shutdown
+    var fflush = Module["_fflush"];
+    if (fflush) fflush(0);
+    var buffers = SYSCALLS.buffers;
+    if (buffers[1].length) SYSCALLS.printChar(1, {{{ charCode("\n") }}});
+    if (buffers[2].length) SYSCALLS.printChar(2, {{{ charCode("\n") }}});
+  },
+  fd_write__deps: ['$flush_NO_FILESYSTEM'],
+#if EXIT_RUNTIME == 1 && !MINIMAL_RUNTIME // MINIMAL_RUNTIME does not have __ATEXIT__ (so it does not get flushed stdout at program exit - programs in MINIMAL_RUNTIME do not have a concept of exiting)
+  fd_write__postset: '__ATEXIT__.push(flush_NO_FILESYSTEM);',
+#endif
+#endif
+  fd_write: function(stream, iov, iovcnt, pnum) {
+#if SYSCALLS_REQUIRE_FILESYSTEM
+    stream = FS.getStream(stream);
+    if (!stream) throw new FS.ErrnoError({{{ cDefine('EBADF') }}});
+    var num = SYSCALLS.doWritev(stream, iov, iovcnt);
+#else
+    // hack to support printf in SYSCALLS_REQUIRE_FILESYSTEM=0
+    var num = 0;
+    for (var i = 0; i < iovcnt; i++) {
+      var ptr = {{{ makeGetValue('iov', 'i*8', 'i32') }}};
+      var len = {{{ makeGetValue('iov', 'i*8 + 4', 'i32') }}};
+      for (var j = 0; j < len; j++) {
+        SYSCALLS.printChar(stream, HEAPU8[ptr+j]);
+      }
+      num += len;
+    }
+#endif // SYSCALLS_REQUIRE_FILESYSTEM
+    {{{ makeSetValue('pnum', 0, 'num', 'i32') }}}
+    return 0;
+  },
+  // Fallback for cases where the wasi_unstable.name prefixing fails,
+  // and we have the full name from C. This happens in fastcomp (which
+  // lacks the attribute to set the import module and base names) and
+  // in LTO mode (as bitcode does not preserve them).
+  __wasi_fd_write: 'fd_write',
 };
 
 if (SYSCALL_DEBUG) {
@@ -1812,17 +1823,29 @@ if (SYSCALL_DEBUG) {
   }
 }
 
+var WASI_SYSCALLS = set(['fd_write']);
+
 for (var x in SyscallsLibrary) {
+  var which = null; // if this is a musl syscall, its number
   var m = /^__syscall(\d+)$/.exec(x);
-  if (!m) continue;
-  var which = +m[1];
+  if (m) {
+    which = +m[1];
+  } else if (!(x in WASI_SYSCALLS)) {
+    continue;
+  }
   var t = SyscallsLibrary[x];
   if (typeof t === 'string') continue;
   t = t.toString();
   var pre = '', post = '';
-  pre += 'SYSCALLS.varargs = varargs;\n';
+  if (which) {
+    pre += 'SYSCALLS.varargs = varargs;\n';
+  }
 #if SYSCALL_DEBUG
-  pre += "err('syscall! ' + [" + which + ", '" + SYSCALL_CODE_TO_NAME[which] + "']);\n";
+  if (which) {
+    pre += "err('syscall! ' + [" + which + ", '" + SYSCALL_CODE_TO_NAME[which] + "']);\n";
+  } else {
+    pre += "err('syscall! " + x + "');\n";
+  }
   pre += "var canWarn = true;\n";
   pre += "var ret = (function() {\n";
   post += "})();\n";
