@@ -56,7 +56,9 @@ var asm2wasmImports = { // special asm2wasm imports
         return x % y;
     },
     "debugger": function() {
+#if ASSERTIONS // Disable debugger; statement from being present in release builds to avoid Firefox deoptimizations, see https://bugzilla.mozilla.org/show_bug.cgi?id=1538375
         debugger;
+#endif
     }
 #if NEED_ALL_ASM2WASM_IMPORTS
     ,
@@ -312,6 +314,42 @@ function loadDynamicLibrary(lib, flags) {
 }
 
 #if WASM
+// Applies relocations to exported things.
+function relocateExports(exports, memoryBase, tableBase, moduleLocal) {
+  var relocated = {};
+
+  for (var e in exports) {
+    var value = exports[e];
+    if (typeof value === 'object') {
+      // a breaking change in the wasm spec, globals are now objects
+      // https://github.com/WebAssembly/mutable-global/issues/1
+      value = value.value;
+    }
+    if (typeof value === 'number') {
+      // relocate it - modules export the absolute value, they can't relocate before they export
+#if EMULATE_FUNCTION_POINTER_CASTS
+      // it may be a function pointer
+      if (e.substr(0, 3) == 'fp$' && typeof exports[e.substr(3)] === 'function') {
+        value = value + tableBase;
+      } else {
+#endif
+        value = value + memoryBase;
+#if EMULATE_FUNCTION_POINTER_CASTS
+      }
+#endif
+    }
+    relocated[e] = value;
+    if (moduleLocal) {
+#if WASM_BACKEND
+      moduleLocal['_' + e] = value;
+#else
+      moduleLocal[e] = value;
+#endif
+    }
+  }
+  return relocated;
+}
+
 // Loads a side module from binary data
 function loadWebAssemblyModule(binary, flags) {
   var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
@@ -403,8 +441,9 @@ function loadWebAssemblyModule(binary, flags) {
       sym = '_' + sym;
 #endif
       var resolved = Module[sym];
-      if (!resolved)
+      if (!resolved) {
         resolved = moduleLocal[sym];
+      }
 #if ASSERTIONS
       assert(resolved, 'missing linked ' + type + ' `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
 #endif
@@ -461,7 +500,7 @@ function loadWebAssemblyModule(binary, flags) {
           return obj[prop] = function() {
             if (!fp) {
               var f = resolveSymbol(name, 'function');
-              fp = addFunctionWasm(f, sig);
+              fp = addFunction(f, sig);
             }
             return fp;
           };
@@ -496,7 +535,6 @@ function loadWebAssemblyModule(binary, flags) {
 #endif
 
     function postInstantiation(instance, moduleLocal) {
-      var exports = {};
 #if ASSERTIONS
       // the table should be unchanged
       assert(table === originalTable);
@@ -510,33 +548,7 @@ function loadWebAssemblyModule(binary, flags) {
         assert(table.get(tableBase + i) !== undefined, 'table entry was not filled in');
       }
 #endif
-      for (var e in instance.exports) {
-        var value = instance.exports[e];
-        if (typeof value === 'object') {
-          // a breaking change in the wasm spec, globals are now objects
-          // https://github.com/WebAssembly/mutable-global/issues/1
-          value = value.value;
-        }
-        if (typeof value === 'number') {
-          // relocate it - modules export the absolute value, they can't relocate before they export
-#if EMULATE_FUNCTION_POINTER_CASTS
-          // it may be a function pointer
-          if (e.substr(0, 3) == 'fp$' && typeof instance.exports[e.substr(3)] === 'function') {
-            value = value + tableBase;
-          } else {
-#endif
-            value = value + memoryBase;
-#if EMULATE_FUNCTION_POINTER_CASTS
-          }
-#endif
-        }
-        exports[e] = value;
-#if WASM_BACKEND
-        moduleLocal['_' + e] = value;
-#else
-        moduleLocal[e] = value;
-#endif
-      }
+      var exports = relocateExports(instance.exports, memoryBase, tableBase, moduleLocal);
       // initialize the module
       var init = exports['__post_instantiate'];
       if (init) {
@@ -649,7 +661,7 @@ var functionPointers = new Array({{{ RESERVED_FUNCTION_POINTERS }}});
 function convertJsFunctionToWasm(func, sig) {
 #if WASM2JS
   return func;
-#endif
+#else // WASM2JS
 
   // The module is static, with the exception of the type section, which is
   // generated based on the signature passed in.
@@ -709,6 +721,7 @@ function convertJsFunctionToWasm(func, sig) {
   });
   var wrappedFunc = instance.exports.f;
   return wrappedFunc;
+#endif // WASM2JS
 }
 
 // Add a wasm function to the table.
@@ -750,10 +763,13 @@ function removeFunctionWasm(index) {
 // 'sig' parameter is required for the llvm backend but only when func is not
 // already a WebAssembly function.
 function addFunction(func, sig) {
+#if ASSERTIONS
+  assert(typeof func !== 'undefined');
 #if ASSERTIONS == 2
   if (typeof sig === 'undefined') {
     err('warning: addFunction(): You should provide a wasm function signature string as a second argument. This is not necessary for asm.js and asm2wasm, but can be required for the LLVM wasm backend, so it is recommended for full portability.');
   }
+#endif // ASSERTIONS == 2
 #endif // ASSERTIONS
 
 #if WASM_BACKEND
