@@ -1,6 +1,15 @@
 /*
-WebAssembly SIMD128 Intrinsics
-*/
+ * Copyright 2019 The Emscripten Authors.  All rights reserved.
+ * Emscripten is available under two separate licenses, the MIT license and the
+ * University of Illinois/NCSA Open Source License.  Both these licenses can be
+ * found in the LICENSE file.
+ */
+
+/*
+ * WebAssembly SIMD128 Intrinsics
+ */
+
+#pragma once
 
 #include <stdint.h>
 #include <stdbool.h>
@@ -607,7 +616,7 @@ static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u16x8_sub_saturate(v128_t a, v1
   return (v128_t)__builtin_wasm_sub_saturate_u_i16x8((__i16x8)a, (__i16x8)b);
 }
 
-// v128_t wasm_i16x8_mul(v128_t a v128_t b)
+// v128_t wasm_i16x8_mul(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_mul(v128_t a, v128_t b) {
   return (v128_t)((__u16x8)a * (__u16x8)b);
 }
@@ -642,17 +651,17 @@ static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u32x4_shr(v128_t a, int32_t b)
   return (v128_t)((__u32x4)a >> b);
 }
 
-// v128_t wasm_i32x4_add(v128_t a v128_t b)
+// v128_t wasm_i32x4_add(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_add(v128_t a, v128_t b) {
   return (v128_t)((__u32x4)a + (__u32x4)b);
 }
 
-// v128_t wasm_i32x4_sub(v128_t a v128_t b)
+// v128_t wasm_i32x4_sub(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_sub(v128_t a, v128_t b) {
   return (v128_t)((__u32x4)a - (__u32x4)b);
 }
 
-// v128_t wasm_i32x4_mul(v128_t a v128_t b)
+// v128_t wasm_i32x4_mul(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_mul(v128_t a, v128_t b) {
   return (v128_t)((__u32x4)a * (__u32x4)b);
 }
@@ -689,12 +698,12 @@ static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u64x2_shr(v128_t a, int32_t b)
   return (v128_t)((__u64x2)a >> (int64_t)b);
 }
 
-// v128_t wasm_i64x2_add(v128_t a v128_t b)
+// v128_t wasm_i64x2_add(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i64x2_add(v128_t a, v128_t b) {
   return (v128_t)((__u64x2)a + (__u64x2)b);
 }
 
-// v128_t wasm_i64x2_sub(v128_t a v128_t b)
+// v128_t wasm_i64x2_sub(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i64x2_sub(v128_t a, v128_t b) {
   return (v128_t)((__u64x2)a - (__u64x2)b);
 }
@@ -716,32 +725,42 @@ static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_sqrt(v128_t a) {
   return (v128_t)__builtin_wasm_sqrt_f32x4((__f32x4)a);
 }
 
-// v128_t wasm_f32x4_add(v128_t a v128_t b)
+// v128_t wasm_f32x4_qfma(v128_t a, v128_t b, v128_t c)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_qfma(v128_t a, v128_t b, v128_t c) {
+  return (v128_t) __builtin_wasm_qfma_f32x4((__f32x4)a, (__f32x4)b, (__f32x4)c);
+}
+
+// v128_t wasm_f32x4_qfms(v128_t a, v128_t b, v128_t c)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_qfms(v128_t a, v128_t b, v128_t c) {
+  return (v128_t) __builtin_wasm_qfms_f32x4((__f32x4)a, (__f32x4)b, (__f32x4)c);
+}
+
+// v128_t wasm_f32x4_add(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_add(v128_t a, v128_t b) {
   return (v128_t)((__f32x4)a + (__f32x4)b);
 }
 
-// v128_t wasm_f32x4_sub(v128_t a v128_t b)
+// v128_t wasm_f32x4_sub(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_sub(v128_t a, v128_t b) {
   return (v128_t)((__f32x4)a - (__f32x4)b);
 }
 
-// v128_t wasm_f32x4_mul(v128_t a v128_t b)
+// v128_t wasm_f32x4_mul(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_mul(v128_t a, v128_t b) {
   return (v128_t)((__f32x4)a * (__f32x4)b);
 }
 
-// v128_t wasm_f32x4_div(v128_t a v128_t b)
+// v128_t wasm_f32x4_div(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_div(v128_t a, v128_t b) {
   return (v128_t)((__f32x4)a / (__f32x4)b);
 }
 
-// v128_t wasm_f32x4_min(v128_t a v128_t b)
+// v128_t wasm_f32x4_min(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_min(v128_t a, v128_t b) {
   return (v128_t)__builtin_wasm_min_f32x4((__f32x4)a, (__f32x4)b);
 }
 
-// v128_t wasm_f32x4_max(v128_t a v128_t b)
+// v128_t wasm_f32x4_max(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_max(v128_t a, v128_t b) {
   return (v128_t)__builtin_wasm_max_f32x4((__f32x4)a, (__f32x4)b);
 }
@@ -763,83 +782,158 @@ static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_sqrt(v128_t a) {
   return (v128_t)__builtin_wasm_sqrt_f64x2((__f64x2)a);
 }
 
-// v128_t wasm_f64x2_add(v128_t a v128_t b)
+// v128_t wasm_f64x2_qfma(v128_t a, v128_t b, v128_t c)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_qfma(v128_t a, v128_t b, v128_t c) {
+  return (v128_t) __builtin_wasm_qfma_f64x2((__f64x2)a, (__f64x2)b, (__f64x2)c);
+}
+
+// v128_t wasm_f64x2_qfms(v128_t a, v128_t b, v128_t c)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_qfms(v128_t a, v128_t b, v128_t c) {
+  return (v128_t) __builtin_wasm_qfms_f64x2((__f64x2)a, (__f64x2)b, (__f64x2)c);
+}
+
+// v128_t wasm_f64x2_add(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_add(v128_t a, v128_t b) {
   return (v128_t)((__f64x2)a + (__f64x2)b);
 }
 
-// v128_t wasm_f64x2_sub(v128_t a v128_t b)
+// v128_t wasm_f64x2_sub(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_sub(v128_t a, v128_t b) {
   return (v128_t)((__f64x2)a - (__f64x2)b);
 }
 
-// v128_t wasm_f64x2_mul(v128_t a v128_t b)
+// v128_t wasm_f64x2_mul(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_mul(v128_t a, v128_t b) {
   return (v128_t)((__f64x2)a * (__f64x2)b);
 }
 
-// v128_t wasm_f64x2_div(v128_t a v128_t b)
+// v128_t wasm_f64x2_div(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_div(v128_t a, v128_t b) {
   return (v128_t)((__f64x2)a / (__f64x2)b);
 }
 
-// v128_t wasm_f64x2_min(v128_t a v128_t b)
+// v128_t wasm_f64x2_min(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_min(v128_t a, v128_t b) {
   return (v128_t)__builtin_wasm_min_f64x2((__f64x2)a, (__f64x2)b);
 }
 
-// v128_t wasm_f64x2_max(v128_t a v128_t b)
+// v128_t wasm_f64x2_max(v128_t a, v128_t b)
 static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_max(v128_t a, v128_t b) {
   return (v128_t)__builtin_wasm_max_f64x2((__f64x2)a, (__f64x2)b);
 }
 
 #endif // __wasm_unimplemented_simd128__
 
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_trunc_saturate_i32x4_f32x4(v128_t a) {
+// v128_t wasm_i32x4_trunc_saturate_f32x4(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_trunc_saturate_f32x4(v128_t a) {
   return (v128_t)__builtin_wasm_trunc_saturate_s_i32x4_f32x4((__f32x4)a);
 }
 
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_trunc_saturate_u32x4_f32x4(v128_t a) {
+// v128_t wasm_u32x4_trunc_saturate_f32x4(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u32x4_trunc_saturate_f32x4(v128_t a) {
   return (v128_t)__builtin_wasm_trunc_saturate_u_i32x4_f32x4((__f32x4)a);
 }
 
 #ifdef __wasm_unimplemented_simd128__
 
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_trunc_saturate_i64x2_f64x2(v128_t a) {
+// v128_t wasm_i64x2_trunc_saturate_f32x4(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i64x2_trunc_saturate_f64x2(v128_t a) {
   return (v128_t)__builtin_wasm_trunc_saturate_s_i64x2_f64x2((__f64x2)a);
 }
 
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_trunc_saturate_u64x2_f64x2(v128_t a) {
+// v128_t wasm_u64x2_trunc_saturate_f64x2(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u64x2_trunc_saturate_f64x2(v128_t a) {
   return (v128_t)__builtin_wasm_trunc_saturate_s_i64x2_f64x2((__f64x2)a);
 }
 
 #endif // __wasm_unimplemented_simd128__
 
-// v128_t wasm_convert_f32x4_i32x4(v128_t a)
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_convert_f32x4_i32x4(v128_t v) {
-  return (v128_t) __builtin_convertvector((__i32x4)v, __f32x4);
+// v128_t wasm_f32x4_convert_i32x4(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_convert_i32x4(v128_t a) {
+  return (v128_t) __builtin_convertvector((__i32x4)a, __f32x4);
 }
 
-// v128_t wasm_convert_f32x4_u32x4(v128_t a)
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_convert_f32x4_u32x4(v128_t v) {
-  return (v128_t) __builtin_convertvector((__u32x4)v, __f32x4);
+// v128_t wasm_f32x4_convert_u32x4(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f32x4_convert_u32x4(v128_t a) {
+  return (v128_t) __builtin_convertvector((__u32x4)a, __f32x4);
 }
 
 #ifdef __wasm_unimplemented_simd128__
 
-// v128_t wasm_convert_f64x2_i64x2(v128_t a)
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_convert_f64x2_i64x2(v128_t v) {
-  return (v128_t) __builtin_convertvector((__i64x2)v, __f64x2);
+// v128_t wasm_f64x2_convert_i64x2(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_convert_i64x2(v128_t a) {
+  return (v128_t) __builtin_convertvector((__i64x2)a, __f64x2);
 }
 
-// v128_t wasm_convert_f64x2_u64x2(v128_t a)
-static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_convert_f64x2_u64x2(v128_t v) {
-  return (v128_t) __builtin_convertvector((__u64x2)v, __f64x2);
+// v128_t wasm_f64x2_convert_u64x2(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_f64x2_convert_u64x2(v128_t a) {
+  return (v128_t) __builtin_convertvector((__u64x2)a, __f64x2);
 }
 
 #endif // __wasm_unimplemented_simd128__
 
+// v128_t wasm_v8x16_shuffle(v128_t a, v128_t b, c0, ..., c15)
 #define wasm_v8x16_shuffle(                                                                        \
   a, b, c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15)                      \
   ((v128_t)(__builtin_shufflevector((__u8x16)(a), (__u8x16)(b), c0, c1, c2, c3, c4, c5, c6, c7,    \
     c8, c9, c10, c11, c12, c13, c14, c15)))
+
+// v128_t wasm_i8x16_narrow_i16x8(v128_t a, v128_t b)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i8x16_narrow_i16x8(v128_t a, v128_t b) {
+  return (v128_t) __builtin_wasm_narrow_s_i8x16_i16x8((__i16x8)a, (__i16x8)b);
+}
+
+// v128_t wasm_u8x16_narrow_i16x8(v128_t a, v128_t b)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u8x16_narrow_i16x8(v128_t a, v128_t b) {
+  return (v128_t) __builtin_wasm_narrow_u_i8x16_i16x8((__i16x8)a, (__i16x8)b);
+}
+
+// v128_t wasm_i16x8_narrow_i32x4(v128_t a, v128_t b)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_narrow_i32x4(v128_t a, v128_t b) {
+  return (v128_t) __builtin_wasm_narrow_s_i16x8_i32x4((__i32x4)a, (__i32x4)b);
+}
+
+// v128_t wasm_u16x8_narrow_i32x4(v128_t a, v128_t b)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_u16x8_narrow_i32x4(v128_t a, v128_t b) {
+  return (v128_t) __builtin_wasm_narrow_u_i16x8_i32x4((__i32x4)a, (__i32x4)b);
+}
+
+// v128_t wasm_i16x8_widen_low_i8x16(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_widen_low_i8x16(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_low_s_i16x8_i8x16((__i8x16)a);
+}
+
+// v128_t wasm_i16x8_widen_high_i8x16(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_widen_high_i8x16(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_high_s_i16x8_i8x16((__i8x16)a);
+}
+
+// v128_t wasm_i16x8_widen_low_u8x16(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_widen_low_u8x16(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_low_u_i16x8_i8x16((__i8x16)a);
+}
+
+// v128_t wasm_i16x8_widen_high_u8x16(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i16x8_widen_high_u8x16(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_high_u_i16x8_i8x16((__i8x16)a);
+}
+
+// v128_t wasm_i32x4_widen_low_i16x8(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_widen_low_i16x8(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_low_s_i32x4_i16x8((__i16x8)a);
+}
+
+// v128_t wasm_i32x4_widen_high_i16x8(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_widen_high_i16x8(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_high_s_i32x4_i16x8((__i16x8)a);
+}
+
+// v128_t wasm_i32x4_widen_low_u16x8(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_widen_low_u16x8(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_low_u_i32x4_i16x8((__i16x8)a);
+}
+
+// v128_t wasm_i32x4_widen_high_u16x8(v128_t a)
+static __inline__ v128_t __DEFAULT_FN_ATTRS wasm_i32x4_widen_high_u16x8(v128_t a) {
+  return (v128_t) __builtin_wasm_widen_high_u_i32x4_i16x8((__i16x8)a);
+}
